/// <reference types="urijs" />
import { StringInput } from "../support/Support";
import { TypeBuilder } from "../TypeBuilder";
import { TypeAttributes } from "../TypeAttributes";
import { JSONSchema, JSONSchemaStore } from "./JSONSchemaStore";
import { RunContext } from "../Run";
import { Input } from "./Inputs";
export declare enum PathElementKind {
    Root = 0,
    KeyOrIndex = 1,
    Type = 2,
    Object = 3,
}
export declare type PathElement = {
    kind: PathElementKind.Root;
} | {
    kind: PathElementKind.KeyOrIndex;
    key: string;
} | {
    kind: PathElementKind.Type;
    index: number;
} | {
    kind: PathElementKind.Object;
};
export declare class Ref {
    readonly path: ReadonlyArray<PathElement>;
    static root(address: string | undefined): Ref;
    private static parsePath(path);
    static parseURI(uri: uri.URI, destroyURI?: boolean): Ref;
    static parse(ref: string): Ref;
    addressURI: uri.URI | undefined;
    constructor(addressURI: uri.URI | undefined, path: ReadonlyArray<PathElement>);
    readonly hasAddress: boolean;
    readonly address: string;
    readonly isRoot: boolean;
    private pushElement(pe);
    push(...keys: string[]): Ref;
    pushObject(): Ref;
    pushType(index: number): Ref;
    resolveAgainst(base: Ref | undefined): Ref;
    readonly name: string;
    readonly definitionName: string | undefined;
    toString(): string;
    private lookup(local, path, root);
    lookupRef(root: JSONSchema): JSONSchema;
    equals(other: any): boolean;
    hashCode(): number;
}
export declare const schemaTypeDict: {
    null: boolean;
    boolean: boolean;
    string: boolean;
    integer: boolean;
    number: boolean;
    array: boolean;
    object: boolean;
};
export declare type JSONSchemaType = keyof typeof schemaTypeDict;
export declare type JSONSchemaAttributes = {
    forType?: TypeAttributes;
    forUnion?: TypeAttributes;
    forObject?: TypeAttributes;
    forCases?: TypeAttributes[];
};
export declare type JSONSchemaAttributeProducer = (schema: JSONSchema, canonicalRef: Ref, types: Set<JSONSchemaType>, unionCases: JSONSchema[] | undefined) => JSONSchemaAttributes | undefined;
export interface JSONSchemaSourceData {
    name: string;
    uris?: string[];
    schema?: StringInput;
    isConverted?: boolean;
}
export declare class JSONSchemaInput implements Input<JSONSchemaSourceData> {
    private _schemaStore;
    private readonly _additionalSchemaAddresses;
    readonly kind: string;
    readonly needSchemaProcessing: boolean;
    private readonly _attributeProducers;
    private readonly _schemaInputs;
    private _schemaSources;
    private readonly _topLevels;
    private _needIR;
    constructor(_schemaStore: JSONSchemaStore | undefined, additionalAttributeProducers?: JSONSchemaAttributeProducer[], _additionalSchemaAddresses?: ReadonlyArray<string>);
    readonly needIR: boolean;
    addTopLevel(name: string, ref: Ref): void;
    addTypes(ctx: RunContext, typeBuilder: TypeBuilder): Promise<void>;
    addSource(schemaSource: JSONSchemaSourceData): Promise<void>;
    finishAddingInputs(): Promise<void>;
    singleStringSchemaSource(): string | undefined;
}
